const shell = require("shelljs");

const commands = [
  // Start middleware
  {
    enabled: process.env.XEDITOR_ENABLE_MIDDLEWARE !== "false",
    cwd: '/usr/app/env/middleware',
    cmd: 'npm run start'
  },
  // May start mock-server
  {
    enabled: process.env.XEDITOR_ENABLE_MOCK_SERVER  !== "false",
    cwd: '/usr/app/env/mock-server',
    cmd: 'npm run start'
  },
  {
    enabled: true,
    cwd: '/usr/app',
    cmd: 'nginx -g "daemon off;"'
  }
];

async function execute(config) {
  if (!config.enabled) {
    return Promise.resolve();
  }

  return new Promise((resolve, reject) => {
    var childProcess = shell.exec(config.cmd, {
      async: true,
      silent: false,
      cwd: config.cwd
    });
    
    const onExit = (code) => {
      if (code === 0) {
        return resolve();
      } else {
        var error = new Error(`Exited with code ${code}`);
        error.code = code;
        return reject(error);
      }
    }
    childProcess.on('close', onExit);
    childProcess.on('exit', onExit);
  });
}

(async () => {
  const processes = [];
  for (const command of commands) {
    processes.push(execute(command));
  }

  await Promise.all(processes).catch((error) => {
    console.log(error);
    process.exit(error.code || 1);
  });
})();